﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  [Item("Rational polynomial (3d)", "")]
  public class RationalPolynomial : RegressionBenchmark {

    public RationalPolynomial() {
      Name = "Rational polynomial (3d)";
      TargetVariable = "y";
      Inputvariables = new Dictionary<string, IntRange>() {
        {"x1", new IntRange(0, 2)},
        {"x2", new IntRange(1, 2)},
        {"x3", new IntRange(1, 2)}
      };
      AmountOfPoints = 4000;
      TrainingPartition = new IntRange(0, 2667);
      TestPartition = new IntRange(TrainingPartition.End, AmountOfPoints);
    }

    #region cloning
    protected RationalPolynomial(RationalPolynomial original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RationalPolynomial(this, cloner);
    }
    #endregion

    protected override double CalculateFunction(Dictionary<string, IList<double>> data, List<string> vars) {
      double x1 = data[vars.ElementAt(0)].Last();
      double x2 = data[vars.ElementAt(1)].Last();
      double x3 = data[vars.ElementAt(2)].Last();
      return (30 * (x1 - 1) * (x3 - 1)) / ((x1 - 10) * Math.Pow(x2, 2));
    }
  }
}
